"""
依赖注入模块
提供容器和提供者的统一管理
"""
from .container import Container, get_container, reset_container
from .providers import ServiceProvider

__all__ = [
    "Container",
    "get_container",
    "reset_container",
    "ServiceProvider",
]
