"""
类型定义模块
提供统一的结果格式和类型定义
"""
from .result import Result, OperationResult, IndexResult, QueryResult
from .storage_types import StorageBackend, StorageConfig

__all__ = [
    "Result",
    "OperationResult",
    "IndexResult",
    "QueryResult",
    "StorageBackend",
    "StorageConfig",
]
