"""
存储策略模块
提供策略模式实现，支持多种存储后端热切换
"""
from .base import StorageStrategy
from .local_strategy import LocalStorageStrategy
from .qdrant_strategy import QdrantStorageStrategy
from .storage_provider import StorageProvider

__all__ = [
    "StorageStrategy",
    "LocalStorageStrategy",
    "QdrantStorageStrategy",
    "StorageProvider",
]
